@@ -38,12 +38,14 @@ import <- function(module, as = NULL, convert = TRUE, delay_load = FALSE) {
   }
   
   # resolve delay load
+  delay_load_environment <- NULL
   delay_load_priority <- 0
   delay_load_functions <- NULL
   if (is.function(delay_load)) {
     delay_load_functions <- list(on_load = delay_load)
     delay_load <- TRUE
   } else if (is.list(delay_load)) {
+    delay_load_environment <- delay_load$environment
     delay_load_functions <- delay_load
     if (!is.null(delay_load$priority))
       delay_load_priority <- delay_load$priority
@@ -64,6 +66,7 @@ import <- function(module, as = NULL, convert = TRUE, delay_load = FALSE) {
   else {
     if (is.null(.globals$delay_load_module) || (delay_load_priority > .globals$delay_load_priority)) {
       .globals$delay_load_module <- module
+      .globals$delay_load_environment <- delay_load_environment
       .globals$delay_load_priority <- delay_load_priority
     }
     module_proxy <- new.env(parent = emptyenv())
@@ -484,7 +487,7 @@ dict <- function(..., convert = FALSE) {
       else
         name
     } else {
-      if (grepl("[0-9]+", name))
+      if (grepl("^[0-9]+$", name))
         name <- as.integer(name)
       else
         name
@@ -725,6 +728,24 @@ py_call <- function(x, ...) {
 }
 
 
+#' Check if a Python object has an attribute
+#'
+#' Check whether a Python object \code{x} has an attribute
+#' \code{name}.
+#' 
+#' @param x A python object.
+#' @param name The attribute to be accessed.
+#'
+#' @return \code{TRUE} if the object has the attribute \code{name}, and
+#'   \code{FALSE} otherwise.
+#' @export
+py_has_attr <- function(x, name) {
+  ensure_python_initialized()
+  if (py_is_module_proxy(x))
+    py_resolve_module_proxy(x)
+  py_has_attr_impl(x, name)
+}
+
 #' Get an attribute of a Python object
 #'
 #' @param x Python object
@@ -736,6 +757,8 @@ py_call <- function(x, ...) {
 #' @export
 py_get_attr <- function(x, name, silent = FALSE) {
   ensure_python_initialized()
+  if (py_is_module_proxy(x))
+    py_resolve_module_proxy(x)
   py_get_attr_impl(x, name, silent)
 }
 
@@ -748,6 +771,8 @@ py_get_attr <- function(x, name, silent = FALSE) {
 #' @export
 py_set_attr <- function(x, name, value) {
   ensure_python_initialized()
+  if (py_is_module_proxy(x))
+    py_resolve_module_proxy(x)
   py_set_attr_impl(x, name, value)
 }
 
@@ -760,6 +785,8 @@ py_set_attr <- function(x, name, value) {
 #' @export
 py_list_attributes <- function(x) {
   ensure_python_initialized()
+  if (py_is_module_proxy(x))
+    py_resolve_module_proxy(x)
   py_list_attributes_impl(x)
 }
 
@@ -1092,6 +1119,7 @@ py_resolve_module_proxy <- function(proxy) {
   
   # clear the global tracking of delay load modules
   .globals$delay_load_module <- NULL
+  .globals$delay_load_environment <- NULL
   .globals$delay_load_priority <- 0
   
   # call on_load if specifed
