@@ -56,3 +56,41 @@ test_that("iter_next returns sentinel value when it completes", {
   expect_equal(item, NA)
 })
 
+sequence_generator <-function(start, completed = NULL) {
+  value <- start
+  function() {
+    value <<- value + 1L
+    gc()
+    if (value < 20L)
+      value
+    else
+      completed
+  }
+}
+
+test_that("generator functions iterate as expected", {
+  skip_if_no_python()
+  gen <- py_iterator(sequence_generator(10L))
+  expect_equal(iterate(gen), 11L:19L)
+})
+
+
+test_that("generator functions support a custom completed value", {
+  skip_if_no_python()
+  gen <- py_iterator(sequence_generator(10L, completed = NA), completed = NA)
+  expect_equal(iterate(gen), 11L:19L)
+})
+
+test_that("generator functions are always called on the main thread", {
+  skip_if_no_python()
+  gen <- py_iterator(sequence_generator(10L))
+  expect_equal(test$iterateOnThread(gen), 11L:19L)
+})
+
+
+
+
+
+
+
+
